==> ponyc-0.21.2-1-x86_64.pkg.tar.xz: usr/share/doc/pony/stdlib-docs/docs/buffered-Reader.md <==
# Reader

Store network data and provide a parsing interface.

`Reader` provides a way to extract typed data from a sequence of
bytes. The `Reader` manages the underlying data structures to
provide a read cursor over a contiguous sequence of bytes. It is
useful for decoding data that is received over a network or stored
in a file. Chunk of bytes are added to the `Reader` using the
`append` method, and typed data is extracted using the getter
methods.

For example, suppose we have a UDP-based network data protocol where
messages consist of the following:

* `list_size` - the number of items in the following list of items
  as a big-endian 32-bit integer
* zero or more items of the following data:
  * a big-endian 64-bit floating point number
  * a string that starts with a big-endian 32-bit integer that
    specifies the length of the string, followed by a number of
    bytes that represent the string

A message would be something like this:

```
[message_length][list_size][float1][string1][float2][string2]...
```

The following program uses a `Reader` to decode a message of
this type and print them:

```pony
use "buffered"
use "collections"

class Notify is StdinNotify
  let _env: Env
  new create(env: Env) =>
    _env = env
  fun ref apply(data: Array[U8] iso) =>
    let rb = Reader
    rb.append(consume data)
    try
      while true do
        let len = rb.i32_be()?
        let items = rb.i32_be()?.usize()
        for range in Range(0, items) do
          let f = rb.f32_be()?
          let str_len = rb.i32_be()?.usize()
          let str = String.from_array(rb.block(str_len)?)
          _env.out.print("[(" + f.string() + "), (" + str + ")]")
        end
      end
    end

actor Main
  new create(env: Env) =>
    env.input(recover Notify(env) end, 1024)
```


```pony
class ref Reader
```

## Constructors

### create

```pony
new iso create()
: Reader iso^
```

#### Returns

* [Reader](buffered-Reader) iso^

---

## Public Functions

### size

Return the number of available bytes.


```pony
fun box size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### clear

Discard all pending data.


```pony
fun ref clear()
: None val
```

#### Returns

* [None](builtin-None) val

---

### append

Add a chunk of data.


```pony
fun ref append(
  data: (String val | Array[U8 val] val))
: None val
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

#### Returns

* [None](builtin-None) val

---

### skip

Skip n bytes.


```pony
fun ref skip(
  n: USize val)
: None val ?
```
#### Parameters

*   n: [USize](builtin-USize) val

#### Returns

* [None](builtin-None) val ?

---

### block

Return a block as a contiguous chunk of memory.
Will throw an error if you request a block larger than what is currently
stored in the `Reader`.


```pony
fun ref block(
  len: USize val)
: Array[U8 val] iso^ ?
```
#### Parameters

*   len: [USize](builtin-USize) val

#### Returns

* [Array](builtin-Array)\[[U8](builtin-U8) val\] iso^ ?

---

### read_until

Find the first occurrence of the separator and return the block of bytes
before its position. The separator is not included in the returned array,
but it is removed from the buffer. To read a line of text, prefer line()
that handles \n and \r\n.


```pony
fun ref read_until(
  separator: U8 val)
: Array[U8 val] iso^ ?
```
#### Parameters

*   separator: [U8](builtin-U8) val

#### Returns

* [Array](builtin-Array)\[[U8](builtin-U8) val\] iso^ ?

---

### line

Return a \n or \r\n terminated line as a string. The newline is not
included in the returned string, but it is removed from the network buffer.


```pony
fun ref line()
: String val ?
```

#### Returns

* [String](builtin-String) val ?

---

### u8

Get a U8. Raise an error if there isn't enough data.


```pony
fun ref u8()
: U8 val ?
```

#### Returns

* [U8](builtin-U8) val ?

---

### i8

Get an I8.


```pony
fun ref i8()
: I8 val ?
```

#### Returns

* [I8](builtin-I8) val ?

---

### u16_be

Get a big-endian U16.


```pony
fun ref u16_be()
: U16 val ?
```

#### Returns

* [U16](builtin-U16) val ?

---

### u16_le

Get a little-endian U16.


```pony
fun ref u16_le()
: U16 val ?
```

#### Returns

* [U16](builtin-U16) val ?

---

### i16_be

Get a big-endian I16.


```pony
fun ref i16_be()
: I16 val ?
```

#### Returns

* [I16](builtin-I16) val ?

---

### i16_le

Get a little-endian I16.


```pony
fun ref i16_le()
: I16 val ?
```

#### Returns

* [I16](builtin-I16) val ?

---

### u32_be

Get a big-endian U32.


```pony
fun ref u32_be()
: U32 val ?
```

#### Returns

* [U32](builtin-U32) val ?

---

### u32_le

Get a little-endian U32.


```pony
fun ref u32_le()
: U32 val ?
```

#### Returns

* [U32](builtin-U32) val ?

---

### i32_be

Get a big-endian I32.


```pony
fun ref i32_be()
: I32 val ?
```

#### Returns

* [I32](builtin-I32) val ?

---

### i32_le

Get a little-endian I32.


```pony
fun ref i32_le()
: I32 val ?
```

#### Returns

* [I32](builtin-I32) val ?

---

### u64_be

Get a big-endian U64.


```pony
fun ref u64_be()
: U64 val ?
```

#### Returns

* [U64](builtin-U64) val ?

---

### u64_le

Get a little-endian U64.


```pony
fun ref u64_le()
: U64 val ?
```

#### Returns

* [U64](builtin-U64) val ?

---

### i64_be

Get a big-endian I64.


```pony
fun ref i64_be()
: I64 val ?
```

#### Returns

* [I64](builtin-I64) val ?

---

### i64_le

Get a little-endian I64.


```pony
fun ref i64_le()
: I64 val ?
```

#### Returns

* [I64](builtin-I64) val ?

---

### u128_be

Get a big-endian U128.


```pony
fun ref u128_be()
: U128 val ?
```

#### Returns

* [U128](builtin-U128) val ?

---

### u128_le

Get a little-endian U128.


```pony
fun ref u128_le()
: U128 val ?
```

#### Returns

* [U128](builtin-U128) val ?

---

### i128_be

Get a big-endian I129.


```pony
fun ref i128_be()
: I128 val ?
```

#### Returns

* [I128](builtin-I128) val ?

---

### i128_le

Get a little-endian I128.


```pony
fun ref i128_le()
: I128 val ?
```

#### Returns

* [I128](builtin-I128) val ?

---

### f32_be

Get a big-endian F32.


```pony
fun ref f32_be()
: F32 val ?
```

#### Returns

* [F32](builtin-F32) val ?

---

### f32_le

Get a little-endian F32.


```pony
fun ref f32_le()
: F32 val ?
```

#### Returns

* [F32](builtin-F32) val ?

---

### f64_be

Get a big-endian F64.


```pony
fun ref f64_be()
: F64 val ?
```

#### Returns

* [F64](builtin-F64) val ?

---

### f64_le

Get a little-endian F64.


```pony
fun ref f64_le()
: F64 val ?
```

#### Returns

* [F64](builtin-F64) val ?

---

### peek_u8

Peek at a U8 at the given offset. Raise an error if there isn't enough
data.


```pony
fun box peek_u8(
  offset: USize val = seq)
: U8 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U8](builtin-U8) val ?

---

### peek_i8

Peek at an I8.


```pony
fun box peek_i8(
  offset: USize val = seq)
: I8 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I8](builtin-I8) val ?

---

### peek_u16_be

Peek at a big-endian U16.


```pony
fun box peek_u16_be(
  offset: USize val = seq)
: U16 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U16](builtin-U16) val ?

---

### peek_u16_le

Peek at a little-endian U16.


```pony
fun box peek_u16_le(
  offset: USize val = seq)
: U16 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U16](builtin-U16) val ?

---

### peek_i16_be

Peek at a big-endian I16.


```pony
fun box peek_i16_be(
  offset: USize val = seq)
: I16 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I16](builtin-I16) val ?

---

### peek_i16_le

Peek at a little-endian I16.


```pony
fun box peek_i16_le(
  offset: USize val = seq)
: I16 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I16](builtin-I16) val ?

---

### peek_u32_be

Peek at a big-endian U32.


```pony
fun box peek_u32_be(
  offset: USize val = seq)
: U32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U32](builtin-U32) val ?

---

### peek_u32_le

Peek at a little-endian U32.


```pony
fun box peek_u32_le(
  offset: USize val = seq)
: U32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U32](builtin-U32) val ?

---

### peek_i32_be

Peek at a big-endian I32.


```pony
fun box peek_i32_be(
  offset: USize val = seq)
: I32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I32](builtin-I32) val ?

---

### peek_i32_le

Peek at a little-endian I32.


```pony
fun box peek_i32_le(
  offset: USize val = seq)
: I32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I32](builtin-I32) val ?

---

### peek_u64_be

Peek at a big-endian U64.


```pony
fun box peek_u64_be(
  offset: USize val = seq)
: U64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U64](builtin-U64) val ?

---

### peek_u64_le

Peek at a little-endian U64.


```pony
fun box peek_u64_le(
  offset: USize val = seq)
: U64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U64](builtin-U64) val ?

---

### peek_i64_be

Peek at a big-endian I64.


```pony
fun box peek_i64_be(
  offset: USize val = seq)
: I64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I64](builtin-I64) val ?

---

### peek_i64_le

Peek at a little-endian I64.


```pony
fun box peek_i64_le(
  offset: USize val = seq)
: I64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I64](builtin-I64) val ?

---

### peek_u128_be

Peek at a big-endian U128.


```pony
fun box peek_u128_be(
  offset: USize val = seq)
: U128 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U128](builtin-U128) val ?

---

### peek_u128_le

Peek at a little-endian U128.


```pony
fun box peek_u128_le(
  offset: USize val = seq)
: U128 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U128](builtin-U128) val ?

---

### peek_i128_be

Peek at a big-endian I129.


```pony
fun box peek_i128_be(
  offset: USize val = seq)
: I128 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I128](builtin-I128) val ?

---

### peek_i128_le

Peek at a little-endian I128.


```pony
fun box peek_i128_le(
  offset: USize val = seq)
: I128 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I128](builtin-I128) val ?

---

### peek_f32_be

Peek at a big-endian F32.


```pony
fun box peek_f32_be(
  offset: USize val = seq)
: F32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [F32](builtin-F32) val ?

---

### peek_f32_le

Peek at a little-endian F32.


```pony
fun box peek_f32_le(
  offset: USize val = seq)
: F32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [F32](builtin-F32) val ?

---

### peek_f64_be

Peek at a big-endian F64.


```pony
fun box peek_f64_be(
  offset: USize val = seq)
: F64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [F64](builtin-F64) val ?

---

### peek_f64_le

Peek at a little-endian F64.


```pony
fun box peek_f64_le(
  offset: USize val = seq)
: F64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [F64](builtin-F64) val ?

---

## Private Functions

### _byte

Get a single byte.


```pony
fun ref _byte()
: U8 val ?
```

#### Returns

* [U8](builtin-U8) val ?

---

### _peek_byte

Get the byte at the given offset without moving the cursor forward.
Raise an error if the given offset is not yet available.


```pony
fun box _peek_byte(
  offset: USize val = seq)
: U8 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U8](builtin-U8) val ?

---

### _distance_of

Get the distance to the first occurrence of the given byte


```pony
fun ref _distance_of(
  byte: U8 val)
: USize val ?
```
#### Parameters

*   byte: [U8](builtin-U8) val

#### Returns

* [USize](builtin-USize) val ?

---

### _search_length

Get the length of a pending line. Raise an error if there is no pending
line.


```pony
fun ref _search_length()
: USize val ?
```

#### Returns

* [USize](builtin-USize) val ?

---

==> ponyc-0.22.3-1-x86_64.pkg.tar.xz: usr/share/doc/pony/stdlib-docs/docs/buffered-Reader.md <==
# Reader
<span class="source-link">[[Source]](src/buffered/reader.md#L3)</span>

Store network data and provide a parsing interface.

`Reader` provides a way to extract typed data from a sequence of
bytes. The `Reader` manages the underlying data structures to
provide a read cursor over a contiguous sequence of bytes. It is
useful for decoding data that is received over a network or stored
in a file. Chunk of bytes are added to the `Reader` using the
`append` method, and typed data is extracted using the getter
methods.

For example, suppose we have a UDP-based network data protocol where
messages consist of the following:

* `list_size` - the number of items in the following list of items
  as a big-endian 32-bit integer
* zero or more items of the following data:
  * a big-endian 64-bit floating point number
  * a string that starts with a big-endian 32-bit integer that
    specifies the length of the string, followed by a number of
    bytes that represent the string

A message would be something like this:

```
[message_length][list_size][float1][string1][float2][string2]...
```

The following program uses a `Reader` to decode a message of
this type and print them:

```pony
use "buffered"
use "collections"

class Notify is InputNotify
  let _env: Env
  new create(env: Env) =>
    _env = env
  fun ref apply(data: Array[U8] iso) =>
    let rb = Reader
    rb.append(consume data)
    try
      while true do
        let len = rb.i32_be()?
        let items = rb.i32_be()?.usize()
        for range in Range(0, items) do
          let f = rb.f32_be()?
          let str_len = rb.i32_be()?.usize()
          let str = String.from_array(rb.block(str_len)?)
          _env.out.print("[(" + f.string() + "), (" + str + ")]")
        end
      end
    end

actor Main
  new create(env: Env) =>
    env.input(recover Notify(env) end, 1024)
```


```pony
class ref Reader
```

## Constructors

### create
<span class="source-link">[[Source]](src/buffered/reader.md#L3)</span>


```pony
new iso create()
: Reader iso^
```

#### Returns

* [Reader](buffered-Reader) iso^

---

## Public Functions

### size
<span class="source-link">[[Source]](src/buffered/reader.md#L69)</span>


Return the number of available bytes.


```pony
fun box size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### clear
<span class="source-link">[[Source]](src/buffered/reader.md#L75)</span>


Discard all pending data.


```pony
fun ref clear()
: None val
```

#### Returns

* [None](builtin-None) val

---

### append
<span class="source-link">[[Source]](src/buffered/reader.md#L82)</span>


Add a chunk of data.


```pony
fun ref append(
  data: (String val | Array[U8 val] val))
: None val
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

#### Returns

* [None](builtin-None) val

---

### skip
<span class="source-link">[[Source]](src/buffered/reader.md#L95)</span>


Skip n bytes.


```pony
fun ref skip(
  n: USize val)
: None val ?
```
#### Parameters

*   n: [USize](builtin-USize) val

#### Returns

* [None](builtin-None) val ?

---

### block
<span class="source-link">[[Source]](src/buffered/reader.md#L121)</span>


Return a block as a contiguous chunk of memory.
Will throw an error if you request a block larger than what is currently
stored in the `Reader`.


```pony
fun ref block(
  len: USize val)
: Array[U8 val] iso^ ?
```
#### Parameters

*   len: [USize](builtin-USize) val

#### Returns

* [Array](builtin-Array)\[[U8](builtin-U8) val\] iso^ ?

---

### read_until
<span class="source-link">[[Source]](src/buffered/reader.md#L161)</span>


Find the first occurrence of the separator and return the block of bytes
before its position. The separator is not included in the returned array,
but it is removed from the buffer. To read a line of text, prefer line()
that handles \n and \r\n.


```pony
fun ref read_until(
  separator: U8 val)
: Array[U8 val] iso^ ?
```
#### Parameters

*   separator: [U8](builtin-U8) val

#### Returns

* [Array](builtin-Array)\[[U8](builtin-U8) val\] iso^ ?

---

### line
<span class="source-link">[[Source]](src/buffered/reader.md#L172)</span>


Return a \n or \r\n terminated line as a string. The newline is not
included in the returned string, but it is removed from the network buffer.


```pony
fun ref line()
: String val ?
```

#### Returns

* [String](builtin-String) val ?

---

### u8
<span class="source-link">[[Source]](src/buffered/reader.md#L207)</span>


Get a U8. Raise an error if there isn't enough data.


```pony
fun ref u8()
: U8 val ?
```

#### Returns

* [U8](builtin-U8) val ?

---

### i8
<span class="source-link">[[Source]](src/buffered/reader.md#L217)</span>


Get an I8.


```pony
fun ref i8()
: I8 val ?
```

#### Returns

* [I8](builtin-I8) val ?

---

### u16_be
<span class="source-link">[[Source]](src/buffered/reader.md#L223)</span>


Get a big-endian U16.


```pony
fun ref u16_be()
: U16 val ?
```

#### Returns

* [U16](builtin-U16) val ?

---

### u16_le
<span class="source-link">[[Source]](src/buffered/reader.md#L233)</span>


Get a little-endian U16.


```pony
fun ref u16_le()
: U16 val ?
```

#### Returns

* [U16](builtin-U16) val ?

---

### i16_be
<span class="source-link">[[Source]](src/buffered/reader.md#L243)</span>


Get a big-endian I16.


```pony
fun ref i16_be()
: I16 val ?
```

#### Returns

* [I16](builtin-I16) val ?

---

### i16_le
<span class="source-link">[[Source]](src/buffered/reader.md#L249)</span>


Get a little-endian I16.


```pony
fun ref i16_le()
: I16 val ?
```

#### Returns

* [I16](builtin-I16) val ?

---

### u32_be
<span class="source-link">[[Source]](src/buffered/reader.md#L255)</span>


Get a big-endian U32.


```pony
fun ref u32_be()
: U32 val ?
```

#### Returns

* [U32](builtin-U32) val ?

---

### u32_le
<span class="source-link">[[Source]](src/buffered/reader.md#L265)</span>


Get a little-endian U32.


```pony
fun ref u32_le()
: U32 val ?
```

#### Returns

* [U32](builtin-U32) val ?

---

### i32_be
<span class="source-link">[[Source]](src/buffered/reader.md#L275)</span>


Get a big-endian I32.


```pony
fun ref i32_be()
: I32 val ?
```

#### Returns

* [I32](builtin-I32) val ?

---

### i32_le
<span class="source-link">[[Source]](src/buffered/reader.md#L281)</span>


Get a little-endian I32.


```pony
fun ref i32_le()
: I32 val ?
```

#### Returns

* [I32](builtin-I32) val ?

---

### u64_be
<span class="source-link">[[Source]](src/buffered/reader.md#L287)</span>


Get a big-endian U64.


```pony
fun ref u64_be()
: U64 val ?
```

#### Returns

* [U64](builtin-U64) val ?

---

### u64_le
<span class="source-link">[[Source]](src/buffered/reader.md#L297)</span>


Get a little-endian U64.


```pony
fun ref u64_le()
: U64 val ?
```

#### Returns

* [U64](builtin-U64) val ?

---

### i64_be
<span class="source-link">[[Source]](src/buffered/reader.md#L307)</span>


Get a big-endian I64.


```pony
fun ref i64_be()
: I64 val ?
```

#### Returns

* [I64](builtin-I64) val ?

---

### i64_le
<span class="source-link">[[Source]](src/buffered/reader.md#L313)</span>


Get a little-endian I64.


```pony
fun ref i64_le()
: I64 val ?
```

#### Returns

* [I64](builtin-I64) val ?

---

### u128_be
<span class="source-link">[[Source]](src/buffered/reader.md#L319)</span>


Get a big-endian U128.


```pony
fun ref u128_be()
: U128 val ?
```

#### Returns

* [U128](builtin-U128) val ?

---

### u128_le
<span class="source-link">[[Source]](src/buffered/reader.md#L329)</span>


Get a little-endian U128.


```pony
fun ref u128_le()
: U128 val ?
```

#### Returns

* [U128](builtin-U128) val ?

---

### i128_be
<span class="source-link">[[Source]](src/buffered/reader.md#L339)</span>


Get a big-endian I129.


```pony
fun ref i128_be()
: I128 val ?
```

#### Returns

* [I128](builtin-I128) val ?

---

### i128_le
<span class="source-link">[[Source]](src/buffered/reader.md#L345)</span>


Get a little-endian I128.


```pony
fun ref i128_le()
: I128 val ?
```

#### Returns

* [I128](builtin-I128) val ?

---

### f32_be
<span class="source-link">[[Source]](src/buffered/reader.md#L351)</span>


Get a big-endian F32.


```pony
fun ref f32_be()
: F32 val ?
```

#### Returns

* [F32](builtin-F32) val ?

---

### f32_le
<span class="source-link">[[Source]](src/buffered/reader.md#L357)</span>


Get a little-endian F32.


```pony
fun ref f32_le()
: F32 val ?
```

#### Returns

* [F32](builtin-F32) val ?

---

### f64_be
<span class="source-link">[[Source]](src/buffered/reader.md#L363)</span>


Get a big-endian F64.


```pony
fun ref f64_be()
: F64 val ?
```

#### Returns

* [F64](builtin-F64) val ?

---

### f64_le
<span class="source-link">[[Source]](src/buffered/reader.md#L369)</span>


Get a little-endian F64.


```pony
fun ref f64_le()
: F64 val ?
```

#### Returns

* [F64](builtin-F64) val ?

---

### peek_u8
<span class="source-link">[[Source]](src/buffered/reader.md#L393)</span>


Peek at a U8 at the given offset. Raise an error if there isn't enough
data.


```pony
fun box peek_u8(
  offset: USize val = seq)
: U8 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U8](builtin-U8) val ?

---

### peek_i8
<span class="source-link">[[Source]](src/buffered/reader.md#L400)</span>


Peek at an I8.


```pony
fun box peek_i8(
  offset: USize val = seq)
: I8 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I8](builtin-I8) val ?

---

### peek_u16_be
<span class="source-link">[[Source]](src/buffered/reader.md#L406)</span>


Peek at a big-endian U16.


```pony
fun box peek_u16_be(
  offset: USize val = seq)
: U16 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U16](builtin-U16) val ?

---

### peek_u16_le
<span class="source-link">[[Source]](src/buffered/reader.md#L412)</span>


Peek at a little-endian U16.


```pony
fun box peek_u16_le(
  offset: USize val = seq)
: U16 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U16](builtin-U16) val ?

---

### peek_i16_be
<span class="source-link">[[Source]](src/buffered/reader.md#L418)</span>


Peek at a big-endian I16.


```pony
fun box peek_i16_be(
  offset: USize val = seq)
: I16 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I16](builtin-I16) val ?

---

### peek_i16_le
<span class="source-link">[[Source]](src/buffered/reader.md#L424)</span>


Peek at a little-endian I16.


```pony
fun box peek_i16_le(
  offset: USize val = seq)
: I16 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I16](builtin-I16) val ?

---

### peek_u32_be
<span class="source-link">[[Source]](src/buffered/reader.md#L430)</span>


Peek at a big-endian U32.


```pony
fun box peek_u32_be(
  offset: USize val = seq)
: U32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U32](builtin-U32) val ?

---

### peek_u32_le
<span class="source-link">[[Source]](src/buffered/reader.md#L436)</span>


Peek at a little-endian U32.


```pony
fun box peek_u32_le(
  offset: USize val = seq)
: U32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U32](builtin-U32) val ?

---

### peek_i32_be
<span class="source-link">[[Source]](src/buffered/reader.md#L442)</span>


Peek at a big-endian I32.


```pony
fun box peek_i32_be(
  offset: USize val = seq)
: I32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I32](builtin-I32) val ?

---

### peek_i32_le
<span class="source-link">[[Source]](src/buffered/reader.md#L448)</span>


Peek at a little-endian I32.


```pony
fun box peek_i32_le(
  offset: USize val = seq)
: I32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I32](builtin-I32) val ?

---

### peek_u64_be
<span class="source-link">[[Source]](src/buffered/reader.md#L454)</span>


Peek at a big-endian U64.


```pony
fun box peek_u64_be(
  offset: USize val = seq)
: U64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U64](builtin-U64) val ?

---

### peek_u64_le
<span class="source-link">[[Source]](src/buffered/reader.md#L460)</span>


Peek at a little-endian U64.


```pony
fun box peek_u64_le(
  offset: USize val = seq)
: U64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U64](builtin-U64) val ?

---

### peek_i64_be
<span class="source-link">[[Source]](src/buffered/reader.md#L466)</span>


Peek at a big-endian I64.


```pony
fun box peek_i64_be(
  offset: USize val = seq)
: I64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I64](builtin-I64) val ?

---

### peek_i64_le
<span class="source-link">[[Source]](src/buffered/reader.md#L472)</span>


Peek at a little-endian I64.


```pony
fun box peek_i64_le(
  offset: USize val = seq)
: I64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I64](builtin-I64) val ?

---

### peek_u128_be
<span class="source-link">[[Source]](src/buffered/reader.md#L478)</span>


Peek at a big-endian U128.


```pony
fun box peek_u128_be(
  offset: USize val = seq)
: U128 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U128](builtin-U128) val ?

---

### peek_u128_le
<span class="source-link">[[Source]](src/buffered/reader.md#L484)</span>


Peek at a little-endian U128.


```pony
fun box peek_u128_le(
  offset: USize val = seq)
: U128 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U128](builtin-U128) val ?

---

### peek_i128_be
<span class="source-link">[[Source]](src/buffered/reader.md#L490)</span>


Peek at a big-endian I129.


```pony
fun box peek_i128_be(
  offset: USize val = seq)
: I128 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I128](builtin-I128) val ?

---

### peek_i128_le
<span class="source-link">[[Source]](src/buffered/reader.md#L496)</span>


Peek at a little-endian I128.


```pony
fun box peek_i128_le(
  offset: USize val = seq)
: I128 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [I128](builtin-I128) val ?

---

### peek_f32_be
<span class="source-link">[[Source]](src/buffered/reader.md#L502)</span>


Peek at a big-endian F32.


```pony
fun box peek_f32_be(
  offset: USize val = seq)
: F32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [F32](builtin-F32) val ?

---

### peek_f32_le
<span class="source-link">[[Source]](src/buffered/reader.md#L508)</span>


Peek at a little-endian F32.


```pony
fun box peek_f32_le(
  offset: USize val = seq)
: F32 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [F32](builtin-F32) val ?

---

### peek_f64_be
<span class="source-link">[[Source]](src/buffered/reader.md#L514)</span>


Peek at a big-endian F64.


```pony
fun box peek_f64_be(
  offset: USize val = seq)
: F64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [F64](builtin-F64) val ?

---

### peek_f64_le
<span class="source-link">[[Source]](src/buffered/reader.md#L520)</span>


Peek at a little-endian F64.


```pony
fun box peek_f64_le(
  offset: USize val = seq)
: F64 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [F64](builtin-F64) val ?

---

## Private Functions

### _byte
<span class="source-link">[[Source]](src/buffered/reader.md#L375)</span>


Get a single byte.


```pony
fun ref _byte()
: U8 val ?
```

#### Returns

* [U8](builtin-U8) val ?

---

### _peek_byte
<span class="source-link">[[Source]](src/buffered/reader.md#L526)</span>


Get the byte at the given offset without moving the cursor forward.
Raise an error if the given offset is not yet available.


```pony
fun box _peek_byte(
  offset: USize val = seq)
: U8 val ?
```
#### Parameters

*   offset: [USize](builtin-USize) val = seq

#### Returns

* [U8](builtin-U8) val ?

---

### _distance_of
<span class="source-link">[[Source]](src/buffered/reader.md#L549)</span>


Get the distance to the first occurrence of the given byte


```pony
fun ref _distance_of(
  byte: U8 val)
: USize val ?
```
#### Parameters

*   byte: [U8](builtin-U8) val

#### Returns

* [USize](builtin-USize) val ?

---

### _search_length
<span class="source-link">[[Source]](src/buffered/reader.md#L591)</span>


Get the length of a pending line. Raise an error if there is no pending
line.


```pony
fun ref _search_length()
: USize val ?
```

#### Returns

* [USize](builtin-USize) val ?

---
